<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\UserCrudController;
use App\Models\Faculty;
use App\Models\Lecturer;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\UserStoreCrudRequest as StoreRequest;
use App\Http\Requests\UserUpdateCrudRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

/**
 * Class LecturerCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class LecturerCrudController extends CrudController
{
    public $lecturer = null;

    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        parent::setup();
        $this->crud->setModel('App\Models\Lecturer');
        $this->crud->setRoute(backpack_url('lecturers'));
        $this->crud->setEntityNameStrings(__('base.lector_add'), __('base.lectors'));
        $this->data['names'] = [
            'single' => __('base.lector'),
            'plural' => __('base.lectors'),
            'button' => __('base.breadcrumbs')
        ];

        if(!(request()->segment(2) == 'create')){
            $this->lecturer = Lecturer::with('user_attributes')->find(request()->segment(2));
        };


        $this->crud->setColumns([
            [
                'name'  => 'first_name',
                'label' => trans('base.first_name'),
                'type'  => 'text',
            ],
            [
                'name'  => 'last_name',
                'label' => trans('base.last_name'),
                'type'  => 'text',
            ],
            [
                'name'  => 'email',
                'label' => trans('backpack::permissionmanager.email'),
                'type'  => 'email',
            ],
//            [ // n-n relationship (with pivot table)
//                'label'     => trans('backpack::permissionmanager.roles'), // Table column heading
//                'type'      => 'select_multiple',
//                'name'      => 'roles', // the method that defines the relationship in your Model
//                'entity'    => 'roles', // the method that defines the relationship in your Model
//                'attribute' => 'name', // foreign key attribute that is shown to user
//                'model'     => config('permission.models.role'), // foreign key model
//            ],
//            [ // n-n relationship (with pivot table)
//                'label'     => trans('backpack::permissionmanager.extra_permissions'), // Table column heading
//                'type'      => 'select_multiple',
//                'name'      => 'permissions', // the method that defines the relationship in your Model
//                'entity'    => 'permissions', // the method that defines the relationship in your Model
//                'attribute' => 'name', // foreign key attribute that is shown to user
//                'model'     => config('permission.models.permission'), // foreign key model
//            ],
        ]);
        $this->crud->addFilter([
            'name' => 'first_name',
            'type' => 'text',
            'label' => __('base.first_name')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('first_name', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'last_name',
            'type' => 'text',
            'label' => __('base.last_name')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('last_name', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'certificate_number',
            'type' => 'text',
            'label' => __('base.certificate')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value', 'like',   "%$value%");
                });
        });

        $this->crud->addFilter([
            'name' => 'faculty',
            'type' => 'select2',
            'label'=> __('base.faculty')
        ],
            Faculty::all()->pluck('name', 'id')->toArray(),
            function($values) {
                $this->crud->query = $this->crud->query->has('specialisations.faculty')->whereHas('specialisations.faculty', function (Builder $query) use ($values) {
                    $query->where('id',  $values);
            });
        });

        $this->crud->addField([
            'name'  => 'first_name',
            'label' => __('base.first_name'),
            'type'  => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
        ]);
        $this->crud->addField([
            'name' => 'last_name',
            'label' => __('base.last_name'),
            'type' => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
        ]);
//        $this->crud->addField([
//            'name' => 'birth_date',
//            'label' => __('base.birth_date'),
//            'type' => 'date',
//        ]);

        $this->crud->addField([
            'name' => 'certificate_number',
            'label' => __('base.certificate'),
            'type' => 'text',
            'fake' => true,
            'value' => (!empty($this->lecturer) && !empty($certificate = $this->lecturer->user_attributes->where('key', 'certificate_number')->first()))? $certificate->value : null,
        ]);

        $this->crud->addField([
            'label' => __('base.faculty'),
            'type' => 'select2_multiple',
            'name' => 'faculties', // the method that defines the relationship in your Model
            'entity' => 'faculties', // the method that defines the relationship in your Model
            'attribute' => 'name', // foreign key attribute that is shown to user
            'model' => "App\Models\Faculty", // foreign key model
            'pivot' => true,
        ]);

        if(!empty( config('settings.academicDegree') )) {
            $this->crud->addField([ // select_from_array
                'name' => 'academicDegree',
                'label' => __('base.academic_degree'),
                'type' => 'select2_from_array',
                'value' => (!empty($this->lecturer) && !empty($academicDegree = Arr::pluck( $this->lecturer->user_attributes->where('key', 'academicDegree')->all(), 'value')))? $academicDegree : null,
                'options' =>  Arr::pluck(json_decode(config('settings.academicDegree'), true), 'name', 'slug'),
                'allows_null' => false,
                'allows_multiple' => true,
                'fake' => true
            ]);
        }
        // Авторизация

        $this->crud->addField([
            'name'  => 'email',
            'label' => trans('base.email'),
            'type'  => 'email',
        ]);

        $this->crud->addField([
            'name'  => 'password',
            'label' => trans('backpack::permissionmanager.password'),
            'type'  => 'password',
        ]);

        $this->crud->addField([
            'name'  => 'password_confirmation',
            'label' => trans('backpack::permissionmanager.password_confirmation'),
            'type'  => 'password',
            'fake' => true,
        ]);
    }
    public function store(StoreRequest $request)
    {
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        $redirect_location = parent::storeCrud($request);

        $current_entry = $this->crud->entry;
        $this->crud->entry->assignRole('Lecturer');
        DB::table('model_has_roles')->insert([ //TODO: пересмотреть
            'role_id' => 4,
            'model_type' => 'App\Models\BackpackUser',
            'model_id' => $current_entry->id,
        ]);

        if(!empty($request->get('certificate_number'))){
            $current_entry->user_attributes()->create( [
                'type' => 'lecturer',
                'key' => 'certificate_number',
                'value' => $request->get('certificate_number')
            ]);
        }

        return $redirect_location;
    }

    /**
     * Update the specified resource in the database.
     *
     * @param UpdateRequest $request - type injection used for validation using Requests
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(UpdateRequest $request, Lecturer $lecturer)
    {
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        $lecturer->user_attributes()->delete();
        $attribute_fields = [
            'academicDegree' => $request->get('academicDegree'),
        ];
        $save = [];
        foreach ($attribute_fields as $key => $attribute) {
            if(!empty($attribute)){
                if( $key == 'academicDegree'){
                    foreach ($attribute as $item){
                        $save[] = [
                            'type' => 'lecturer',
                            'key' => $key,
                            'value' => $item
                        ];
                    }
                } else {
                    $save[] = [
                        'type' => 'lecturer',
                        'key' => $key,
                        'value' => $attribute
                    ];
                }

            }
        }
        $lecturer->user_attributes()->createMany($save);

        if(!empty($request->get('certificate_number'))){
            $lecturer->user_attributes()->create( [
                'type' => 'lecturer',
                'key' => 'certificate_number',
                'value' => $request->get('certificate_number')
            ]);
        }

        return parent::updateCrud($request);
    }
    protected function handlePasswordInput(Request $request)
    {
        // Remove fields not present on the user.
        $request->request->remove('password_confirmation');

        // Encrypt password if specified.
        if ($request->input('password')) {
            $request->request->set('password', Hash::make($request->input('password')));
        } else {
            $request->request->remove('password');
        }
    }
}
