<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\MoodleController;
use App\Models\Faculty;
use App\Models\Specialisation;
use App\Models\Student;
use Backpack\CRUD\app\Http\Controllers\CrudController;

// VALIDATION: change the requests to match your own file names if you need form validation
use App\Http\Requests\StudentRequest as StoreRequest;
use App\Http\Requests\StudentRequest as UpdateRequest;
use Backpack\CRUD\CrudPanel;
use Auth;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

/**
 * Class StudentCrudController
 * @package App\Http\Controllers\Admin
 * @property-read CrudPanel $crud
 */
class StudentCrudController extends CrudController
{

    public $student = null;
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        parent::setup();
        $this->crud->setModel('App\Models\Student');
        $this->crud->setRoute(backpack_url('students'));
        $this->crud->setEntityNameStrings(__('base.statement.student_add'), __('base.statement.students'));
        $this->crud->setRequiredFields(StoreRequest::class, 'create');
        $this->crud->setRequiredFields(UpdateRequest::class, 'edit');

        if( $ids = Auth::user()->coordinatorAddClause()){
            $this->crud->addClause('whereHas', 'specialisations', function($query) use ($ids){
                $query->whereHas('faculty', function($query) use ($ids){
                    $query->where('id', $ids);
                });
            });
        }

        $this->data['names'] = [
            'single' => __('base.student'),
            'plural' => __('base.students'),
            'button' => __('base.breadcrumbs')];

        if(!(request()->segment(2) == 'create')){
            $this->student = Student::with('user_attributes')->find(request()->segment(2));
        };

        $this->crud->addButtonFromView('line', 'show_subjects', 'show_subjects', 'beginning');

        if(Auth::user()->hasAnyRole(['SuperAdmin', 'Coordinator'])){
            $this->crud->addButtonFromModelFunction('line', 'print', 'print', 'beginning');
        }

        $this->crud->setColumns([
            [
                'name'  => 'card_id',
                'label' => trans('base.card_id'),
                'type' => "model_function",
                'function_name' => 'getCardId',
            ],
            [
                'name'  => 'first_name',
                'label' => trans('base.first_name'),
                'type'  => 'text',
            ],
            [
                'name'  => 'last_name',
                'label' => trans('base.last_name'),
                'type'  => 'text',
            ],
            [
                'name'  => 'gender',
                'label' => trans('base.gender'),
                'type' => "model_function",
                'limit' => 99999999,
                'function_name' => 'getStudentGender',
            ],
            [
                'name'  => 'birth_date',
                'label' => trans('base.birth_date'),
                'type'  => 'date',
            ],
            [
                'name'  => 'email',
                'label' => trans('base.email'),
                'type'  => 'email',
            ],
            [
                'name'  => 'student_status',
                'label' => trans('base.status'),
                'type' => "model_function",
                'limit' => 99999999,
                'function_name' => 'getStudentStatusView',
            ],
            [
                'name'  => 'student_group_name',
                'label' => trans('base.group.group'),
                'type' => 'text',
            ],
        ]);

        $this->getFields();

        $this->crud->addFilter([
            'name' => 'studentCard',
            'type' => 'text',
            'label' => __('base.card_id')
        ],
        false,
        function ($value) {
            $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                $query->where('value', 'like',  "%$value%");
            });
        });

        $this->crud->addFilter([
            'name' => 'first_name',
            'type' => 'text',
            'label' => __('base.first_name')
        ],
        false,
        function ($value) {
            $this->crud->query = $this->crud->query->where('first_name', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'last_name',
            'type' => 'text',
            'label' => __('base.last_name')
        ],
        false,
        function ($value) {
            $this->crud->query = $this->crud->query->where('last_name', 'like', "%$value%");
        });

        if( config('settings.gender') ){
            $this->crud->addFilter([
                'name' => 'gender',
                'type' => 'dropdown',
                'label' => __('base.gender')
            ],
                Arr::pluck(json_decode(config('settings.gender'), true), 'name', 'slug'),
                function ($value) {
                    $this->crud->addClause('where', 'gender', $value);
                });
        }

        $this->crud->addFilter([
                'type' => 'date_range',
                'name' => 'from_to',
                'label'=> __('base.birth_date')
            ],
            false,
            function($value) {
                $dates = json_decode($value);
                $this->crud->addClause('where', 'birth_date', '>=', $dates->from);
                $this->crud->addClause('where', 'birth_date', '<=', $dates->to);
        });

//        $this->crud->addFilter([
//                'type' => 'date_range',
//                'name' => 'payment_from_to',
//                'label'=> __('base.payment_date')
//            ],
//            false,
//            function($value) {
//                $dates = json_decode($value);
//                $this->crud->addClause('where', 'payment_date', '>=', $dates->from);
//                $this->crud->addClause('where', 'payment_date', '<=', $dates->to);
//        });


        if( config('settings.studyPrograms') ){
            $this->crud->addFilter([
                'name' => 'studyProgram',
                'type' => 'select2',
                'label' => __('base.group.program')
            ],
                Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug'),
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value',  $value);
                });
            });
        }

        if( config('settings.academicDegree') ){
            $this->crud->addFilter([
                'name' => 'academicDegree',
                'type' => 'select2',
                'label' => __('base.academic_degree')
            ],
                Arr::pluck(json_decode(config('settings.academicDegree'), true), 'name', 'slug'),
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value',  $value);
                });
            });
        }
        if( config('settings.statuses') ){
            $this->crud->addFilter([
                'name' => 'status',
                'type' => 'select2',
                'label' => __('base.status')
            ],
                Arr::pluck(json_decode(config('settings.statuses'), true), 'name', 'slug'),
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value',  $value);
                });
            });
        }

        $this->crud->addFilter([
            'name' => 'faculties',
            'type' => 'select2',
            'label'=> __('base.faculty')
        ],
        Faculty::all()->pluck('name', 'id')->toArray(),
        function($values) {
            $this->crud->query = $this->crud->query->has('specialisations.faculty')->whereHas('specialisations.faculty', function (Builder $query) use ($values) {
                $query->where('id',  $values);
            });
        });

        $this->crud->addFilter([
            'name' => 'specialisation',
            'type' => 'select2',
            'label'=> __('base.specialisation')
        ],
        Specialisation::all()->pluck('name', 'id')->toArray(),
        function($values) {
            $this->crud->query = $this->crud->query->whereHas('specialisations', function (Builder $query) use ($values) {
                $query->where('id',  $values);
            });
        });

        $this->crud->addFilter([
            'name' => 'group',
            'type' => 'select2_ajax',
            'label'=> __('base.group.group'),
            'placeholder' => 'Pick a category',
        ],
            backpack_url( 'ajax-groups' ),
            function($values) {
                $this->crud->query = $this->crud->query->whereHas('groups', function (Builder $query) use ($values) {
                    $query->where('id',  $values);
                });
            });

        $this->crud->addFilter([
            'name' => 'email',
            'type' => 'text',
            'label' => __('base.email')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('email','like', "%$value%");
            });

        $this->crud->addFilter([
            'name' => 'passport',
            'type' => 'text',
            'label' => __('base.passport')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value',  $value);
                });
            });
    }
    public function store(StoreRequest $request)
    {
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        $redirect_location = parent::storeCrud($request);

        $current_entry = $this->crud->entry;
        $current_entry->assignRole('Student');
        DB::table('model_has_roles')->insert([ //TODO: пересмотреть
            'role_id' => 5,
            'model_type' => 'App\Models\BackpackUser',
            'model_id' => $current_entry->id,
        ]);

        MoodleController::createUser(
            $current_entry->id,
            $request->get('password'),
            $request->get('first_name'),
            $request->get('last_name'),
            $request->get('email')
        );

        $attributes = [
            'studentCard' => $request->get('studentCard'),
            'citizenship' => $request->get('citizenship'),
            'passport' => $request->get('passport'),
            'phone_number' => $request->get('phone_number'),
            'address' => $request->get('address'),
            'address_actual' => $request->get('address_actual'),
            'studyLanguages' => $request->get('studyLanguages'),
            'studyOrder' => $request->get('studyOrder'),
            'status' => $request->get('status'),
            'studyProgram' => $request->get('studyProgram'),
            'academicDegree' => $request->get('academicDegree'),
            'startDate' => $request->get('startDate'),
            'payments' => $request->get('payments'),
            'semester' => $request->get('semester'),
        ];
        $save = [];
        foreach ($attributes as $key => $attribute) {
            if(!empty($attribute)){
                $save[] = [
                    'type' => 'student',
                    'key' => $key,
                    'value' => $attribute
                ];
            }
        }
        $current_entry->user_attributes()->createMany($save);

        return $redirect_location;
    }

    /**
     * Update the specified resource in the database.
     *
     * @param UpdateRequest $request - type injection used for validation using Requests
     *
     * @param Student $student
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(UpdateRequest $request, Student $student)
    {
        $phone_number = $request->get('phone_number');
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        $student->user_attributes()->delete();
        $attributes = [
            'studentCard' => $request->get('studentCard'),
            'citizenship' => $request->get('citizenship'),
            'passport' => $request->get('passport'),
            'phone_number' => ($phone_number) ? $phone_number : null,
            'address' => $request->get('address'),
            'address_actual' => $request->get('address_actual'),
            'studyLanguages' => $request->get('studyLanguages'),
            'studyOrder' => $request->get('studyOrder'),
            'status' => $request->get('status'),
            'studyProgram' => $request->get('studyProgram'),
            'academicDegree' => $request->get('academicDegree'),
            'startDate' => $request->get('startDate'),
            'payments' => $request->get('payments'),
            'semester' => $request->get('semester'),
        ];
        $save = [];
        foreach ($attributes as $key => $attribute) {
            if(!empty($attribute)){
                $save[] = [
                    'type' => 'student',
                    'key' => $key,
                    'value' => $attribute
                ];
            }
        }
        $student->user_attributes()->createMany($save);

        is_null($request->get('password')) ? $temp_pass = $student->password : $temp_pass = $request->get('password');
        MoodleController::updateUser( $student->id, $request->get('first_name'), $request->get('last_name'), $request->get('email'), $temp_pass);

        return parent::updateCrud($request);
    }

    public function destroy($id)
    {
        $this->crud->hasAccessOrFail('delete');
        $this->crud->setOperation('delete');

        // get entry ID from Request (makes sure its the last ID for nested resources)
        $id = $this->crud->getCurrentEntryId() ?? $id;

//        MoodleController::deleteUser($id);
        return $this->crud->delete($id);
    }

    private function getFields()
    {

        // Персональные данные

        $this->crud->addField([
            'name'  => 'first_name',
            'label' => __('base.first_name'),
            'type'  => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'tab'   => __('base.personal_credentials')
        ]);
        $this->crud->addField([
            'name' => 'last_name',
            'label' => __('base.last_name'),
            'type' => 'text',
            'wrapperAttributes' => [
                'class' => 'form-group col-md-6'
            ],
            'tab'   => __('base.personal_credentials')
        ]);
        $this->crud->addField([
            'name' => 'birth_date',
            'label' => __('base.birth_date'),
            'type' => 'date',
            'tab'   => __('base.personal_credentials')
        ]);

        if(!empty( config('settings.gender') ))
            $this->crud->addField([
                'name' => 'gender',
                'label' => __('base.gender'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.gender'), true), 'name', 'slug'),
                'allows_null' => false,
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-6'
                ],
                'tab' => __('base.personal_credentials')
            ]);

        if(!empty( config('settings.citizenships') ))
            $this->crud->addField([
                'name' => 'citizenship',
                'label' => __('base.citizenship'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.citizenships'), true), 'name', 'slug'),
                'allows_null' => false,
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-6'
                ],
                'tab' => __('base.personal_credentials'),
                'fake' => true,
                //'value' => (!empty($this->student))? $this->student->citizenship : '',
                'value' => (!empty($this->student) && !empty($citizenship = $this->student->user_attributes->where('key', 'citizenship')->first()))? $citizenship->value : null,
            ]);

        $this->crud->addField([
            'name' => 'passport',
            'label' => __('base.passport'),
            'type' => 'text',
            'tab'   => __('base.personal_credentials'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($passport = $this->student->user_attributes->where('key', 'passport')->first()))? $passport->value : null,
        ]);

        $this->crud->addField([
            'name' => 'phone_number',
            'label' => __('base.phone_number'),
            'type' => 'text',
            'tab'   => __('base.personal_credentials'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($passport = $this->student->user_attributes->where('key', 'phone_number')->first()))? $passport->value : null,
        ]);
        $this->crud->addField([
            'name'  => 'address',
            'label' => __('base.address'),
            'type'  => 'text',
            'tab'   => __('base.personal_credentials'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($address = $this->student->user_attributes->where('key', 'address')->first()))? $address->value : null,
        ]);
        $this->crud->addField([
            'name'  => 'address_actual',
            'label' => __('base.address_actual'),
            'type'  => 'text',
            'tab'   => __('base.personal_credentials'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($address_actual = $this->student->user_attributes->where('key', 'address_actual')->first()))? $address_actual->value : null,
        ]);

        // Авторизация

        $this->crud->addField([
            'name'  => 'email',
            'label' => trans('base.email'),
            'type'  => 'email',
            'tab'   => __('base.login')
        ]);

        $this->crud->addField([
            'name'  => 'password',
            'label' => trans('backpack::permissionmanager.password'),
            'type'  => 'password',
            'tab' => __('base.login')
        ]);

        $this->crud->addField([
            'name'  => 'password_confirmation',
            'label' => trans('backpack::permissionmanager.password_confirmation'),
            'type'  => 'password',
            'tab' => __('base.login'),
            'fake' => true,
        ]);

        // Академические данные

        $this->crud->addFields([
            [
                'label' => __('base.card_id'),
                'type' => 'text',
                'name' => 'studentCard',
                'fake' => true,
                'tab'   => __('base.academic_credentials'),
                'value' => (!empty($this->student) && !empty($studentCard = $this->student->user_attributes->where('key', 'studentCard')->first()))? $studentCard->value : null,
            ],[
                'label' => __('base.specialisations'),
                'type' => 'select2_multiple',
                'name' => 'specialisations', // the method that defines the relationship in your Model
                'entity' => 'specialisations', // the method that defines the relationship in your Model
                'attribute' => 'name', // foreign key attribute that is shown to user
                'model' => "App\Models\Specialisation", // foreign key model
                'pivot' => true,
                'options'   => (function ($query) {
                    if(!empty(session('faculty'))) {
                        return $query->whereHas('faculty', function($query){
                            $query->where('id', session('faculty'));
                        })->get();
                    } else {
                        return $query->get();
                    }
                }),
                'tab'   => __('base.academic_credentials')
            ],[
                'label' => __('base.groups'),
                'type' => 'select2_multiple',
                'name' => 'groups', // the method that defines the relationship in your Model
                'entity' => 'groups', // the method that defines the relationship in your Model
                'attribute' => 'name', // foreign key attribute that is shown to user
                'model' => "App\Models\Group", // foreign key model
                'pivot' => true,
                'options'   => (function ($query) {
                    if(!empty(session('faculty'))) {
                        return $query->whereHas('specialisation', function($query){
                            return $query->whereHas('faculty', function($query){
                                $query->where('id', session('faculty'));
                            });
                        })->get();
                    } else {
                        return $query->get();
                    }
                }),
                'tab'   => __('base.academic_credentials')
            ]
        ]);

        if(!empty( config('settings.studyPrograms') ))
            $this->crud->addField([
                'name' => 'studyProgram',
                'label' => __('base.group.program'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.studyPrograms'), true), 'name', 'slug'),
                'allows_null' => false,
                'fake' => true,
                'value' => (!empty($this->student) && !empty($status = $this->student->user_attributes->where('key', 'studyProgram')->first()))? $status->value : null,
                'tab'   => __('base.academic_credentials')

            ]);

        if(!empty( config('settings.studyLanguages') ))
            $this->crud->addField([
                'name' => 'studyLanguages',
                'label' => __('base.study_language'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.studyLanguages'), true), 'name', 'slug'),
                'allows_null' => false,
                'tab' => __('base.academic_credentials'),
                'fake' => true,
                'value' => (!empty($this->student) && !empty($studyLanguages = $this->student->user_attributes->where('key', 'studyLanguages')->first()))? $studyLanguages->value : null,
            ]);

        if(!empty( config('settings.studyOrder') ))
            $this->crud->addField([
                'name' => 'studyOrder',
                'label' => __('base.study_order'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.studyOrder'), true), 'name', 'slug'),
                'allows_null' => false,
                'tab' => __('base.academic_credentials')
            ]);

        if(!empty( config('settings.statuses') ))
            $this->crud->addField([
                'name' => 'status',
                'label' => __('base.status'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.statuses'), true), 'name', 'slug'),
                'allows_null' => false,
                'tab' => __('base.academic_credentials'),
                'fake' => true,
                'value' => (!empty($this->student) && !empty($status = $this->student->user_attributes->where('key', 'status')->first()))? $status->value : null,
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-5'
                ],

        ]);

        if( request()->segment(3) == 'edit'){
            $this->crud->addField([
                'name' => 'extraction',
                'label' => __('base.extractions.label'),
                'type' => 'button',
                'fake' => true,
                'tab'   => __('base.academic_credentials'),
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-2'
                ],
            ]);
        }

        if(!empty( config('settings.semester') ))
            $this->crud->addField([
                'name' => 'semester',
                'label' => __('base.semester'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.semester'), true), 'name', 'slug'),
                'allows_null' => false,
                'tab' => __('base.academic_credentials'),
                'fake' => true,
                'value' => (!empty($this->student) && !empty($status = $this->student->user_attributes->where('key', 'semester')->first()))? $status->value : null,
                'wrapperAttributes' => [
                    'class' => 'form-group col-md-5'
                ],

        ]);

        if(!empty( config('settings.academicDegree') ))
            $this->crud->addField([
                'name' => 'academicDegree',
                'label' => __('base.academic_degree'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.academicDegree'), true), 'name', 'slug'),
                'allows_null' => false,
                'tab' => __('base.academic_credentials'),
                'fake' => true,
                'value' => (!empty($this->student) && !empty($academicDegree = $this->student->user_attributes->where('key', 'academicDegree')->first()))? $academicDegree->value : null,

        ]);

        $this->crud->addField([
            'name' => 'startDate',
            'label' => __('base.start_date'),
            'type' => 'date',
            'tab'   => __('base.academic_credentials'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($startDate = $this->student->user_attributes->where('key', 'startDate')->first()))? $startDate->value : null,
        ]);

        // Платежи
        $this->crud->addField([
            'name' => 'payments',
            'label' => __('base.payments'),
            'type' => 'table',
            'entity_singular' => 'option', // used on the "Add X" button
            'columns' => [
                'amount' => __('base.payment_amount'),
                'debt' => __('base.payment_debt')
            ],
            'min' => 0, // minimum rows allowed in the table
            'tab' => __('base.payments'),
            'fake' => true,
            'value' => (!empty($this->student) && !empty($payments = $this->student->user_attributes->where('key', 'payments')->first()))? $payments->value : null,
        ]);
    }
    protected function handlePasswordInput(Request $request)
    {
        // Remove fields not present on the user.
//        $request->request->remove('password_confirmation');

        // Encrypt password if specified.
        if ($request->input('password')) {
            $this->validate($request, [
                'password' => 'min:6|confirmed',
            ]);
            $request->request->set('password', Hash::make($request->input('password')));
        } else {
            $request->request->remove('password');
        }
    }
}
