<?php

namespace App\Http\Controllers\Admin;

use Backpack\CRUD\app\Http\Controllers\CrudController;
use App\Http\Requests\UserStoreCrudRequest as StoreRequest;
use App\Http\Requests\UserUpdateCrudRequest as UpdateRequest;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Hash;
use Auth;
use Spatie\Permission\Models\Role;

class UserCrudController extends CrudController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            if(!empty(Auth::user()) && !Auth::user()->hasAnyRole(['SuperAdmin'])){
                return back();
            }
            return $next($request);
        });
    }

    public function setup()
    {
        /*
        |--------------------------------------------------------------------------
        | BASIC CRUD INFORMATION
        |--------------------------------------------------------------------------
        */
        $this->crud->setModel(config('backpack.permissionmanager.models.user'));
        $this->crud->setEntityNameStrings(trans('backpack::permissionmanager.user'), trans('backpack::permissionmanager.users'));
        $this->crud->setRoute(backpack_url('users'));

        // Columns.
        $this->crud->setColumns([
            [
                'name'  => 'full_name',
                'label' => trans('base.full_name'),
                'type'  => 'text',
            ],
            [
                'name'  => 'email',
                'label' => trans('backpack::permissionmanager.email'),
                'type'  => 'email',
            ],
            [ // n-n relationship (with pivot table)
               'label'     => trans('backpack::permissionmanager.roles'), // Table column heading
               'type'      => 'select_multiple',
               'name'      => 'roles', // the method that defines the relationship in your Model
               'entity'    => 'roles', // the method that defines the relationship in your Model
               'attribute' => 'name', // foreign key attribute that is shown to user
               'model'     => config('permission.models.role'), // foreign key model
            ],
            [ // n-n relationship (with pivot table)
               'label'     => trans('backpack::permissionmanager.extra_permissions'), // Table column heading
               'type'      => 'select_multiple',
               'name'      => 'permissions', // the method that defines the relationship in your Model
               'entity'    => 'permissions', // the method that defines the relationship in your Model
               'attribute' => 'name', // foreign key attribute that is shown to user
               'model'     => config('permission.models.permission'), // foreign key model
            ],
        ]);

        // Fields
        $this->crud->addFields([
            [
                'name'  => 'first_name',
                'label' => trans('base.first_name'),
                'type'  => 'text',
            ],            [
                'name'  => 'last_name',
                'label' => trans('base.last_name'),
                'type'  => 'text',
            ]
        ]);

        if(!empty( config('settings.gender') )){
            $this->crud->addFields([[
                'name' => 'gender',
                'label' => __('base.gender'),
                'type' => 'select2_from_array',
                'options' => Arr::pluck(json_decode(config('settings.gender'), true), 'name', 'slug'),
                ]
            ]);
        }
        $this->crud->addFields([
            [
                'name'  => 'email',
                'label' => trans('base.email'),
                'type'  => 'email',
            ],
            [
                'label' => __('base.faculty'),
                'type' => 'select2_multiple',
                'name' => 'faculties', // the method that defines the relationship in your Model
                'entity' => 'faculties', // the method that defines the relationship in your Model
                'attribute' => 'name', // foreign key attribute that is shown to user
                'model' => "App\Models\Faculty", // foreign key model
                'pivot' => true,
            ],
            [
                'name'  => 'password',
                'label' => trans('backpack::permissionmanager.password'),
                'type'  => 'password',
            ],
            [
                'name'  => 'password_confirmation',
                'label' => trans('backpack::permissionmanager.password_confirmation'),
                'type'  => 'password',
            ],
            [
            // two interconnected entities
            'label'             => trans('backpack::permissionmanager.user_role_permission'),
            'field_unique_name' => 'user_role_permission',
            'type'              => 'checklist_dependency',
            'name'              => 'roles_and_permissions', // the methods that defines the relationship in your Model
            'subfields'         => [
                    'primary' => [
                        'label'            => trans('backpack::permissionmanager.roles'),
                        'name'             => 'roles', // the method that defines the relationship in your Model
                        'entity'           => 'roles', // the method that defines the relationship in your Model
                        'entity_secondary' => 'permissions', // the method that defines the relationship in your Model
                        'attribute'        => 'name', // foreign key attribute that is shown to user
                        'model'            => config('permission.models.role'), // foreign key model
                        'pivot'            => true, // on create&update, do you need to add/delete pivot table entries?]
                        'number_columns'   => 3, //can be 1,2,3,4,6
                    ],
                    'secondary' => [
                        'label'          => ucfirst(trans('backpack::permissionmanager.permission_singular')),
                        'name'           => 'permissions', // the method that defines the relationship in your Model
                        'entity'         => 'permissions', // the method that defines the relationship in your Model
                        'entity_primary' => 'roles', // the method that defines the relationship in your Model
                        'attribute'      => 'name', // foreign key attribute that is shown to user
                        'model'          => config('permission.models.permission'), // foreign key model
                        'pivot'          => true, // on create&update, do you need to add/delete pivot table entries?]
                        'number_columns' => 3, //can be 1,2,3,4,6
                    ],
                ],
            ],
        ]);

        $this->crud->addFilter([
            'name' => 'first_name',
            'type' => 'text',
            'label' => __('base.first_name')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('first_name', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'last_name',
            'type' => 'text',
            'label' => __('base.last_name')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('last_name', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'email',
            'type' => 'text',
            'label' => __('base.email')
        ],
            false,
            function ($value) {
                $this->crud->query = $this->crud->query->where('email', 'like', "%$value%");
        });

        $this->crud->addFilter([
            'name' => 'studyProgram',
            'type' => 'select2',
            'label' => __('base.role')
        ],
            Role::get()->pluck('name','id')->toArray(),
            function ($value) {
                $this->crud->query = $this->crud->query->whereHas('user_attributes', function (Builder $query) use ($value) {
                    $query->where('value',  $value);
            });
        });
    }

    /**
     * Store a newly created resource in the database.
     *
     * @param StoreRequest $request - type injection used for validation using Requests
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(StoreRequest $request)
    {
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        return parent::storeCrud($request);
    }

    /**
     * Update the specified resource in the database.
     *
     * @param UpdateRequest $request - type injection used for validation using Requests
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(UpdateRequest $request)
    {
        $this->handlePasswordInput($request);

        $request->request->remove('roles_show');
        $request->request->remove('permissions_show');

        return parent::updateCrud($request);
    }

    /**
     * Handle password input fields.
     *
     * @param Request $request
     */
    protected function handlePasswordInput(Request $request)
    {
        // Remove fields not present on the user.
        $request->request->remove('password_confirmation');

        // Encrypt password if specified.
        if ($request->input('password')) {
            $request->request->set('password', Hash::make($request->input('password')));
        } else {
            $request->request->remove('password');
        }
    }
}
