<?php

namespace App\Models;

use App\User;
use Backpack\Base\app\Models\Traits\InheritsRelationsFromParentModel;
use Backpack\Base\app\Notifications\ResetPasswordNotification as ResetPasswordNotification;
use App\Traits\HasTranslations;
use Hash;
use Auth;
use Spatie\Permission\Traits\HasRoles;

class BackpackUser extends User
{
    use InheritsRelationsFromParentModel;
    use HasTranslations;

    protected $table = 'users';

    public $translatable = ['first_name', 'last_name'];

    /**
     * Send the password reset notification.
     *
     * @param string $token
     *
     * @return void
     */
    public function sendPasswordResetNotification($token)
    {
        $this->notify(new ResetPasswordNotification($token));
    }

    /**
     * Get the e-mail address where password reset links are sent.
     *
     * @return string
     */
    public function getEmailForPasswordReset()
    {
        return $this->email;
    }
    
    public function getRoleAttribute(){
        if($this->hasAnyRole('Lecturer')){
            return 'Lecturer';
        } elseif($this->hasAnyRole('Coordinator')){
            return 'Coordinator';
        } elseif($this->hasAnyRole('SuperAdmin')){
            return 'SuperAdmin';
        }
        return 'Student';
    }

    /*
    |--------------------------------------------------------------------------
    | FUNCTIONS
    |--------------------------------------------------------------------------
    */
    protected function asJson($value)
    {
        return json_encode($value, JSON_UNESCAPED_UNICODE);
    }

    protected function facultyIDs()
    {
        $faculties = $this->faculties;
        if ($faculties->isNotEmpty()){
            return $ids = $faculties->pluck('id');
        }
    }

    public function realFacultyIDs()
    {
        $specsIDs = $this->specsIDs();
        if (!empty($specsIDs) && $specsIDs->isNotEmpty()){
            $faculties = \App\Models\Faculty::whereHas('specialisations', function($query) use ($specsIDs){
                $query->whereIn('id', $specsIDs);
            })->get();
            if(!empty($faculties) && $faculties->isNotEmpty()){
                return $faculties->pluck('id');
            }
        }
    }

    protected function specsIDs()
    {
        $specialisations = $this->specialisations();
        // dd($faculties);
        if ($specialisations->isNotEmpty()){
            return $specialisations;
        }
    }

    public static function coordinatorAddClause()
    {
        if(!empty(session('faculty'))) {
            return session('faculty');
        }
        if( !Auth::user()->hasRole('SuperAdmin') && Auth::user()->hasRole('Coordinator')){
            return Auth::user()->facultyIDs();
        }
    }

    /*
    |--------------------------------------------------------------------------
    | RELATIONS
    |--------------------------------------------------------------------------
    */
    public function faculties(){
        return $this->belongsToMany('App\Models\Faculty', 'user_faculty', 'user_id');
    }
    public function specialisations(){

        $u = \DB::table('user_specialisation')->where('user_id', Auth::id())->select('specialisation_id')->get();
        return $u->pluck('specialisation_id');
        
        // return $this->belongsToMany('App\Models\Specialisation', 'user_specialisation', 'user_id');
    }
    public function messages()
    {
        return $this->hasMany('App\Message');
    }
    public function messagesOutcoming()
    {
        return $this->hasMany('App\Message', 'recipient_id');
    }
    public function messagesIncoming()
    {
        return $this->hasMany('App\Message', 'sender_id');
    }
    public function allUserMessages() {
        return $this->messagesOutcoming->merge($this->messagesIncoming);
    }
    /*
    |--------------------------------------------------------------------------
    | SCOPES
    |--------------------------------------------------------------------------
    */
    public function scopeSuperAdmins($query)
    {
        return $query->withRole('SuperAdmin');
    }

    public function scopeAdmins($query)
    {
        return $query->withRole('Admin');
    }

    public function scopeLecturers($query)
    {
        return $query->withRole('Lecturer');
    }

    public function scopeStudents($query)
    {
        return $query->withRole('Student');
    }

    /*
    |--------------------------------------------------------------------------
    | ACCESORS
    |--------------------------------------------------------------------------
    */

    public function getFullNameAttribute()
    {
//        dd($this->getTranslations('first_name'));
        return "{$this->last_name} {$this->first_name}";
    }

    /*
    |--------------------------------------------------------------------------
    | MUTATORS
    |--------------------------------------------------------------------------
    */
    public function setPasswordAttribute($value)
    {
        if(!empty($value))
            $this->attributes['password'] = $value;
    }
}
